#include <iostream>
#include <memory>

#include "raylib.h"

#include "LoopEngine/Global.h"
#include "LoopEngine/GameObject/GOButton.h"
#include "LoopEngine/GameObject/GOAnimated.h"
#include "LoopEngine/GameObject/GOText.h"
#include "LoopEngine/GameObject/GOTextCounter.h"
#include "LoopEngine/GameObject/GOComposite.h"

using namespace std;
int main()
{
    shared_ptr<LE::Engine> engine = InitLoopEngine(1000, 1000, "Demo composite");
    shared_ptr<LE::GOComposite> root = make_shared<LE::GOComposite>("Test");

    {
        auto UIRoot = root->AddNew<LE::GOComposite>("Root UI");
        UIRoot->SetPosition({500.f,100.f});

        auto txtCounterLabel = UIRoot->AddNew<LE::GOText>(0, 100, "Counter: ");
        txtCounterLabel->SetText("Counter: ");

        auto txtCounterValue = UIRoot->AddNew<LE::GOTextCounter>(200, 100);

        auto btnStart = UIRoot->AddNew<LE::GOButton>(-200, 150, 100, 50, "Start",
            [&txtCounterValue]() {
                txtCounterValue->SetActive(true);
            }
        );

        auto btnStop = UIRoot->AddNew<LE::GOButton>(0, 150, 100, 50, "Pause",
            [&txtCounterValue]() {
                txtCounterValue->SetActive(false);
            }
        );

        auto btnDuration = UIRoot->AddNew<LE::GOButton>(200, 150, 100, 50, "Duration",
            [&txtCounterValue]() {
                txtCounterValue->SetDuration((float)GetRandomValue(5, 100) / 100.f);
            }
        );
    }

    const string numberGridImgPath = GetWorkingDirectory() + string("\\res\\AnimGrille4x4.png");
    auto animNumbers = root->AddNew<LE::GOAnimated>(numberGridImgPath, Vector2{ 300.f,500.f }, 4, 4);
    animNumbers->SetFrames({ 0,2,4,6 });

    const string explosionImgPath = GetWorkingDirectory() + string("\\res\\AnimExplosion.png");
    auto animExplosion = root->AddNew<LE::GOAnimated>(explosionImgPath, Vector2{ 700.f,500.f }, 4, 4);
    animExplosion->SetFrameDuration(0.05f);
    animExplosion->SetLooping(false);

    auto anotherExplosion = root->AddNew<LE::GOAnimated>(explosionImgPath, Vector2{ 200.f,200.f }, 4, 4);
    anotherExplosion->SetFrames({ 10,9,8,7,6,5,4,3,2,1,0 });

    if (engine && root)
    {
        engine->RegisterScene(root);
        engine->SetClearColor(GRAY);
        engine->StartGame("Test");
    }
}